/*
 * 2017 Petr Socha
 * Czech Technical University in Prague
 * Faculty of Information Technology
 * Department of Digital Design
 * 
 * This alfa version is intended for internal testing use only!
 */

#include <stdio.h>
#include <stdlib.h>
#include <omp.h>
#include <tclap/CmdLine.h>

char convertAscii2Hex(char asciiChar){    
    if(asciiChar >= 0x30 && asciiChar <= 0x39)
        return asciiChar - 0x30;
    if(asciiChar >= 0x41 && asciiChar <= 0x46)
        return asciiChar - 0x41 + 10;    
    if(asciiChar >= 0x61 && asciiChar <= 0x66)
        return asciiChar - 0x61 + 10;
    return 0;
}

int main(int argc, char * argv[]){
    
    TCLAP::CmdLine cmd("This program is used to convert hex values in ASCII to a binary file.", ' ', "0.1alfa");

    TCLAP::ValueArg<std::string> inputArg("i", "input", "Name of input ascii file", true, "", "Name of an existing file");
    TCLAP::ValueArg<std::string> outputArg("o", "output", "Name of binary file where converted hex values are saved", true, "", "Name of a file to be created");        
    
    cmd.add(outputArg);
    cmd.add(inputArg);
    
    try {
        cmd.parse(argc, argv);
    }
    catch (TCLAP::ArgException &e) {
        std::cerr << "error: " << e.error() << " for arg " << e.argId() << std::endl;
    }

    FILE * ascii_file = fopen(inputArg.getValue().c_str(), "rt");
    FILE * hex_file = fopen(outputArg.getValue().c_str(), "wb");

    if(ascii_file == NULL || hex_file == NULL){
        fprintf(stderr, "Filesystem error: cannot open files.\n");
        return 1;
    }
    
    char * asciiLine = NULL;
    size_t  lineChars = 0;
    
    double start_time = omp_get_wtime();
    
    while(getline(&asciiLine, &lineChars, ascii_file) > 0){
        if(lineChars < 32){
            fprintf(stderr, "Parsing error: line shorter than 128bits.\n");
            return 2;
        }
        for(int i = 0; i < 16; i++)
            fprintf(hex_file, "%c", (convertAscii2Hex(asciiLine[2*i]) << 4) | convertAscii2Hex(asciiLine[2*i+1]));        
    }
    
    double time = omp_get_wtime() - start_time;
    fprintf(stderr, "Job done, %lfs elapsed.\n", time);
    
    fflush(hex_file);
    fclose(hex_file);
    fclose(ascii_file);
    return 0;
}
